﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinListView;

namespace WLV_LoadItemsAsync
{
    class CustomCreationFilter : IUIElementCreationFilter
    {
        public event EventHandler<ItemActivatedEventArgs> ItemInitialized;

        private Appearance folderAppearance = null;
        private Dictionary<string, Appearance> fileAppearanceDictionary;

        public void AfterCreateChildElements(UIElement parent)
        {
            var listViewItemUIElement = parent as UltraListViewItemUIElement;

            // Check if we should procces the item
            if (   listViewItemUIElement == null 
                || listViewItemUIElement.Item == null
                || listViewItemUIElement.Item.Disposed 
                || listViewItemUIElement.Item.HasAppearance)
                return;

            var item = listViewItemUIElement.Item;

            // Check if the item represents directory
            if (Utils.IsDirectory(item.Key))
            {
                var directoryInfo = new DirectoryInfo(item.Key);

                item.SubItems["FileType"].Value = "File Folder";
                item.SubItems["DateModified"].Value = directoryInfo.LastWriteTime;
                item.Appearance = this.FolderAppearance;
            }
            // If it isn't directory, then it is a file
            else
            {
                var fileInfo = new FileInfo(item.Key);
 
                item.SubItems["FileSize"].Value = fileInfo.Length / 1024;
                item.SubItems["FileType"].Value = "File";
                item.SubItems["DateModified"].Value = fileInfo.LastWriteTime;
                // If the extension is not .exe or .lnk (shortcut) we can cache it and use it later
                // If it is .exe it will have unique icon, so we can't use the extension only
                if (fileInfo.Extension != ".exe" && fileInfo.Extension != ".lnk")
                {
                    item.Appearance = GetAppearanceForFile(fileInfo, fileInfo.Extension);
                }
                else
                {
                    item.Appearance = GetAppearanceForFile(fileInfo, fileInfo.FullName);
                }
            }

            // I added an event in case you want to know in the form, when an item is initialized.
            if (ItemInitialized != null)
                ItemInitialized.Invoke(this, new ItemActivatedEventArgs(item));
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }

        private Appearance GetAppearanceForFile(FileInfo fileInfo, string dictionaryKey)
        {
            // Checks if we have cahced the appearance for this file/ extension
            // If we have it uses it, else it creates it
            Appearance appearance;

            this.FileAppearanceDictionary.TryGetValue(dictionaryKey, out appearance);

            if (appearance == null)
            {
                appearance = new Appearance();
                var icon = Icon.ExtractAssociatedIcon(fileInfo.FullName);
                appearance.Image = icon.ToBitmap();

                this.FileAppearanceDictionary.Add(dictionaryKey, appearance);
            }

            return appearance;
        }

        private Appearance FolderAppearance
        {
            get
            {
                if (this.folderAppearance == null)
                {
                    folderAppearance = new Appearance();
                    folderAppearance.Image = Image.FromFile(@"..\..\Resources\Folder.bmp");
                    ((Bitmap)folderAppearance.Image).MakeTransparent(Color.Black);
                }

                return folderAppearance;
            }
            
        }

        public Dictionary<string, Appearance> FileAppearanceDictionary
        {
            get 
            {
                if (this.fileAppearanceDictionary == null)
                    fileAppearanceDictionary = new Dictionary<string, Appearance>();

                return fileAppearanceDictionary; 
            }
        }

        public void ClearCachedInfo()
        {
            // Clears cached items
            if (fileAppearanceDictionary != null)
            {
                foreach (var item in fileAppearanceDictionary.Values)
                {
                    item.Dispose();
                }
                fileAppearanceDictionary.Clear();
            }

            if (folderAppearance != null)
                folderAppearance.Dispose();

        }
    }
}
